<?php class crystal extends baseobject { 
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'prefix'=>validator::REQUIRED,
			'suffix'=>validator::ANY,
			'welldropid'=>array(validator::REQUIRED, validator::INTEGER),
			'dropimageid'=>array(validator::REQUIRED, validator::INTEGER),
			'numberindrop'=>array(validator::REQUIRED, validator::INTEGER),
			'pixelx'=>validator::INTEGER,
			'pixely'=>validator::INTEGER,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this crystal',
			'welldropid'=>'The drop where this crystal grew',
			'dropimageid'=>'The image on which the crystal was marked',
			'numberindrop'=>'The scoring system used for this plate',
			'pixelx'=>'Location of the crosshair in pixels from left of image',
			'pixely'=>'Location of the crosshair in pixels from top of image',
	);

	public static $defaultSortOrder='welldropid ASC, numberindrop ASC';
	
	
// 	protected static $adminSelect='SELECT crystal.*, project.name AS projectname, project.isarchived AS isarchived, 
// 			platetype.name AS platetypename, user.fullname AS ownername
// 			FROM plate, platetype, project, user WHERE project.id=plate.projectid AND platetype.id=plate.platetypeid AND user.id=plate.ownerid';
// 	protected static $normalSelect='SELECT plate.*,project.name AS projectname, project.isarchived AS isarchived, 
// 			platetype.name AS platetypename, user.fullname AS ownername 
// 			FROM plate, platetype, project, user WHERE project.id=plate.projectid AND platetype.id=plate.platetypeid AND user.id=plate.ownerid';
	
	
// 	protected static function getProjectClause($accessType){
// 		return ' AND ((1=1 '.database::getProjectClause($accessType).') OR plate.ownerid='.session::getUserId().')';
// 	}
	
	public static function create($request=array()){
		if(!isset($request['welldropid'])){
			return parent::create($request); //will fail validation and throw correct exception
		}
		$welldrop=welldrop::getbyid($request['welldropid']);
		$platewell=platewell::getbyid($welldrop['platewellid']);
		$plate=plate::getById($platewell['plateid']);
		$rowLabels=array('','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','Q','X','Y','Z');
		$row=$rowLabels[$platewell['row']];
		$col=$platewell['col'];
		if(strlen($col)<2){ $col='0'.$col; }
		$wellLabel=$row.$col;
		$request['name']=$plate['name'].$wellLabel.'d'.$welldrop['dropnumber'].'c'.$request['numberindrop'];
		if(!empty($welldrop['constructid'])){
			$construct=construct::getbyid($welldrop['constructid']);
			$request['name']=$construct['proteinacronym'].'_'.$request['name'];
		}
		$request['prefix']=$request['name'];
		return parent::create($request);
	}	
	
	public static function update($id, $request=array()){
		$crystal=static::getbyid($id);
		if(isset($request['prefix']) || isset($request['suffix'])){
			$newPrefix=$crystal['prefix'];
			$newSuffix=$crystal['suffix'];
			if(isset($request['prefix'])){ $newPrefix=trim($request['prefix']); }
			if(isset($request['suffix'])){ $newSuffix=trim($request['suffix']); }
			if(empty($newSuffix)){
				$request['name']=$newPrefix;
			} else {
				$request['name']=$newPrefix.'_'.$newSuffix;
			}
			if(45<strlen($request['name'])){
				throw new BadRequestException('Name was '.strlen($request['name']).' characters long, but cannot be more than 45 characters (limit imposed by synchrotron ISPyB system).');
			}
		}
		return parent::update($id, $request);
	}
}

?>