<?php class imagingsession extends baseobject {
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'temperature'=>array(validator::REQUIRED, validator::INTEGER),
			'manufacturerdatabaseid'=>array(validator::REQUIRED,validator::INTEGER),
			'imagerid'=>array(validator::REQUIRED, validator::INTEGER),
			'plateid'=>array(validator::REQUIRED, validator::INTEGER),
			'imagingparametersversionid'=>array(validator::REQUIRED, validator::INTEGER),
			'imageddatetime'=>array(validator::REQUIRED, validator::DATETIME),
			'lighttype'=>validator::REQUIRED,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this plate type',
			'temperature'=>'The incubation temperature',
			'manufacturerdatabaseid'=>'The ID of this imaging session in the imager manufacturer database',
			'imagerid'=>'The imaging device used',
			'plateid'=>'The plate being imaged',
			'imagingparametersversionid'=>'The imager settings used for this imaging session',
			'imageddatetime'=>'When this imaging session happened',
			'lighttype'=>'Whether visible or UV light was used',
	);
	
	protected static $adminSelect='SELECT SQL_CALC_FOUND_ROWS imagingsession.*, 
				plate.name AS platename, plate.platetypeid AS platetypeid, plate.ownerid AS plateownerid, user.fullname AS ownername,
				plate.screenid AS screenid, plate.offsetinscreenx AS offsetinscreenx, plate.offsetinscreeny AS offsetinscreeny, 
				plate.crystalscoringsystemid AS crystalscoringsystemid,
				imager.name AS imagername, imager.friendlyname AS imagerfriendlyname, 
				project.name AS projectname, project.isarchived AS isarchived
			FROM imagingsession, plate, imager, project, user
			WHERE project.id=imagingsession.projectid AND plate.id=imagingsession.plateid AND imager.id=imagingsession.imagerid AND user.id=plate.ownerid ';
	protected static $normalSelect='SELECT SQL_CALC_FOUND_ROWS imagingsession.*, 
				plate.name AS platename, plate.platetypeid AS platetypeid, plate.ownerid AS plateownerid, user.fullname AS ownername,
				plate.screenid AS screenid, plate.offsetinscreenx AS offsetinscreenx, plate.offsetinscreeny AS offsetinscreeny, 
				plate.crystalscoringsystemid AS crystalscoringsystemid, 
				imager.name AS imagername, imager.friendlyname AS imagerfriendlyname,
				project.name AS projectname, project.isarchived AS isarchived
			FROM imagingsession, plate, imager, project, user
			WHERE project.id=imagingsession.projectid AND plate.id=imagingsession.plateid AND imager.id=imagingsession.imagerid AND user.id=plate.ownerid ';
	
	protected static function getProjectClause($accessType){
		return ' AND ((1=1 '.database::getProjectClause($accessType).') OR plate.ownerid='.session::getUserId().')';
	}
	
	public static function create($request=array()){
		$plate=plate::getById($request['plateid']);
		if(!$plate){ throw new BadRequestException('Could not find plate with ID '.$request['plateid']); }
		$imager=imager::getById($request['imagerid']);
		if(!$imager){ throw new BadRequestException('Could not find imager with ID '.$request['imagerid']); }
		$request['projectid']=$plate['projectid'];
		$request['temperature']=$imager['temperature'];
		$created=parent::create($request,'imagingsession');
		return $created;
	}
	
	/**
	 * Returns the drop image records for the specified imaging session.
	 * @param unknown $id the ID of the imaging session
	 * @param unknown $request The request parameters, including any pagination or sort order requirements
	 * @return array an array containing the drop image records
	 */
	public static function getdropimages($id,$request=array()){
		return dropimage::getByProperty('imagingsessionid', $id, $request);
	}
	
}

?>