<?php class screen extends baseobject {
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'manufacturer'=>validator::ANY,
			'catalognumber'=>validator::ANY,
			'isstandard'=>validator::BOOLEAN,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this screen',
			'manufacturer'=>'The manufacturer of the screen',
			'catalognumber'=>'Manufacturer\'s catalogue number',
			'isstandard'=>'Whether this is a standard screen (not an optimization)',
	);

	public static $defaultSortOrder='screen.isstandard DESC, name ASC';
	
	private static $rowLabels=array('-','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
	
	private static $csvMimetypes = array(
	    'text/csv',
	    'text/plain',
	    'application/csv',
	    'text/comma-separated-values',
	    'application/excel',
	    'application/vnd.ms-excel',
	    'application/vnd.msexcel',
	    'text/anytext',
	    'application/octet-stream',
	    'application/txt',
	);
	
	public static function canCreateStandardScreens(){
		$projects=session::getCreateProjects();
		if(empty($projects)){ return false; }
		$sharedProject=project::getByName(project::SHARED);
		return in_array($sharedProject['id'], $projects);
	}

	public static function canUpdate($id){
		 if(parent::canUpdate($id)){ return true; }
		 $groupid=usergroup::getByName(usergroup::TECHNICIANS);
		 return usergroup::userisingroup($groupid);
	}
	
	public static function create($request=array()){
		if(isset($request['projectid'])){
			$project=project::getById($request['projectid']);
			$request['isstandard']=false;
		} else {
			$project=project::getByName(project::SHARED);
			$request['projectid']=$project['id'];
			$request['isstandard']=true;
		}
		if(empty($_FILES)){
			$ret=parent::create($request,'screen');
			$screen=$ret['created'];
			$rows=$request['rows'];
			$cols=$request['cols'];
			static::makeDummyScreenConditions($screen, $rows, $cols);
		} else {

			if(!isset($request['name'])){
				$request['name']=$_FILES['file']['name'];
			}
			
			//Parse file for conditions
			$fileContent=file_get_contents($_FILES['file']['tmp_name']);
			if(false!==strpos($fileContent,'Rock Maker XML')){
				$ret=static::parseRockMakerXmlToDatabase($fileContent, $request);
				$screen=$ret['created'];
			} else if(false!==strpos($fileContent, 'crystaltrak ')){
				$ret=static::parseCrystalTrakXmlToDatabase($fileContent, $request);
				$screen=$ret['created'];
			} else if(in_array($_FILES['file']['type'], static::$csvMimetypes)){

				$fc=preg_split("/\r\n|\n|\r/", $fileContent);
				if(3>count($fc)){
					throw new BadRequestException('File does not look like a screen - not enough rows');
				}
				$delimiter=',';
				$thirdRow=str_getcsv($fc[2], $delimiter);
					if(2>count($thirdRow)){
						$delimiter=';';
						$thirdRow=str_getcsv($fileContent[2], $delimiter);
						if(2>count($thirdRow)){ 
							throw new BadRequestException('Could not parse CSV file, tried comma and semicolon as delimiters'); 
						}
					}
				$thirdRow=str_getcsv($fc[2], $delimiter);
				if(2==count($thirdRow)){
					$ret=static::parseWellNameAndConditionToDatabase($fileContent, $request);
					$screen=$ret['created'];
				} else if(2<count($thirdRow)){
					$ret=static::parseMimerCsvToDatabase($fileContent, $request);
					$screen=$ret['created'];
				} else {
					throw new BadRequestException('File does not look like a screen - not enough columns');
				}
				
			} else {
				//Cannot parse file - just attach it and "see file"
				$ret=parent::create($request,'screen');
				$screen=$ret['created'];
				$rows=$request['rows'];
				$cols=$request['cols'];
				static::makeDummyScreenConditionsForUnparseableFile($screen, $rows, $cols);
			}
			//Create DB record of file and save it
			$file=file::create(array(
					'description'=>'Screen definition file',
					'parentid'=>$screen['id'],
			));
				
		}
		return $ret;
	}

	public static function getscreenconditions($id,$request){
		$ret=screencondition::getByProperty('screenid', $id, $request);
		$screen=static::getById($id);
		$ret['screen']=$screen;
		return $ret;
	}
	
	
	private static function makeDummyScreenConditions($screen, $rows, $cols){
		for($r=1;$r<=$rows;$r++){
			for($c=1;$c<=$cols;$c++){
				$wellNumber=(($r-1)*$cols)+$c;
				screencondition::create(array(
						'projectid'=>$screen['projectid'],
						'screenid'=>$screen['id'],
						'name'=>'screen'.$screen['id'].'_well'.str_pad($wellNumber, 3, '0', STR_PAD_LEFT),
						'description'=>'Condition '.static::$rowLabels[$r].str_pad($c, 2, '0', STR_PAD_LEFT),
						'row'=>$r,
						'col'=>$c,
						'wellnumber'=>$wellNumber
				));
			}
		}
	}
	
	
	private static function makeDummyScreenConditionsForUnparseableFile($screen, $rows, $cols){
		for($r=1;$r<=$rows;$r++){
			for($c=1;$c<=$cols;$c++){
				$wellNumber=(($r-1)*$cols)+$c;
				screencondition::create(array(
						'projectid'=>$screen['projectid'],
						'screenid'=>$screen['id'],
						'name'=>'screen'.$screen['id'].'_well'.str_pad($wellNumber, 3, '0', STR_PAD_LEFT),
						'description'=>'See screen definition file',
						'row'=>$r,
						'col'=>$c,
						'wellnumber'=>$wellNumber
				));
			}
		}
	}
	
	
	private static function parseCrystalTrakXmlToDatabase($fileContent, $request){
		$xml=simplexml_load_string($fileContent);
		if(!$xml){ throw new BadRequestException('Detected CrystalTrak XML file but it was not valid XML'); }
		//var_dump($xml->reservoir_design->format->attributes() );
		$format=$xml->reservoir_design->format->attributes();
		if(empty($format['rows']) || empty($format['cols'])){
			throw new Exception('Rows and columns not defined in screen file');
		}
		$request['name']=$xml->reservoir_design->attributes()['name'];
		$request['rows']=$format['rows'];
		$request['cols']=$format['cols'];
		$ret=parent::create($request,'screen');
		$screen=$ret['created'];
		
		static::makeDummyScreenConditions($screen, $format['rows'], $format['cols']);
		foreach($xml->reservoir_design->children() as $well){
			if($well->getName()!="well"){ 
				continue; //format and comments elements are of no interest here
			}
			$description=array();
			$wellNumber=$well->attributes()['number'];
			foreach($well->children() as $item){
				$attrs=$item->attributes();
				$desc=$attrs['class'].' '.$attrs['conc'].$attrs['units'].' '.$attrs['name'];
				if(!empty($attrs['ph'])){ $desc.=', pH '.$attrs['ph']; }
				$description[]=$desc;
			}
			$description=implode('; ', $description);
			$condition=database::queryGetOne(
					'SELECT * FROM screencondition WHERE screenid=:screenid AND wellnumber=:wellnumber', 
					array(':screenid'=>$screen['id'], ':wellnumber'=>$wellNumber)
			);
			$conditionId=$condition['id'];
			screencondition::update($conditionId, array(
					'description'=>$description
			));
			$con=screencondition::getById($conditionId);
		}
		return $ret;
	}

	private static function parseRockMakerXmlToDatabase($fileContent, $request){
		//@TODO Parse RMXML
		throw new BadRequestException('Detected Rock Maker XML file but parsing these files is not supported yet');
		$xml=simplexml_load_string($fileContent);
		if(!$xml){ throw new BadRequestException('Detected Rock Maker XML file but it was not valid XML'); }
		var_dump($xml);
		exit;
	}

	private static function parseMimerCsvToDatabase($fileContent, $request){
		/* Example CSV format: note that the column order may be different and row 1 may not have the numbers.
		Incomplete factorial,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		Reagent number,PEG 8000,[PEG 8000],[PEG 8000] units,pH_PEG 8000,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,"bistris propane, pH7","[bistris propane, pH7]","[bistris propane, pH7] units","pH_bistris propane, pH7","cacodylate, pH 6.5","[cacodylate, pH 6.5]","[cacodylate, pH 6.5] units","pH_cacodylate, pH 6.5",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A1,PEG 8000,8,%,,calcium acetate,0.15,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A2,PEG 8000,8,%,,calcium acetate,0.2,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A3,PEG 8000,8,%,,calcium acetate,0.25,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A4,PEG 8000,8,%,,,,,,calcium acetate,0.15,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A5,PEG 8000,8,%,,,,,,calcium acetate,0.2,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A6,PEG 8000,8,%,,,,,,calcium acetate,0.25,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		B1,PEG 8000,10,%,,calcium acetate,0.15,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		*/
		$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		$delimiter=',';
		$firstRow=str_getcsv($fileContent[0], $delimiter);
		if(2>count($firstRow)){
			$delimiter=';';
			$firstRow=str_getcsv($fileContent[0], $delimiter);
			if(2>count($firstRow)){ 
				throw new BadRequestException('Could not parse CSV file, tried comma and semicolon as delimiters'); 
			}
		}
		if(!isset($request['name'])){
			if(empty($firstRow[0])){
				throw new BadRequestException('No name supplied, and could not find one in cell A1');
			}
			$request['name']=$firstRow[0];			
		}
		
		//Try to create the screen before parsing its conditions
		$ret=parent::create($request,'screen');
		$screenId=$ret['created']['id'];		
		
		$rows=0;
		$cols=0;
		/* Step 1. Parse 2nd row headers for components and their properties.
		 * We aim to create an array where key is name/role and value is an array containing the column numbers for name, amount, units and optionally pH.
		 * Later we use this to know which columns to mash together in what order.
		 * This part is made unnecessarily filthy by the wonderful world of almost-the-same-but-not-quite-just-for-the-hell-of-it screen formats. 
		 */
		$secondRow=str_getcsv($fileContent[1], $delimiter);
		$parts=array();
		//1a. First extract all the unbracketed labels that are not "pH" - these are assumed to be the chemical names/roles
		for($i=1;$i<100;$i++){ //1 because we know 0 is "reagent number" or similar
			if(!isset($secondRow[$i])){ break; }
			$columnCutoff=$i;
			$label=trim($secondRow[$i]);
			if(empty($label)){ break; }
			if(false!==strpos($label,'[') || 'pH'==$label || 0===strpos($label,'pH_')){ continue; }
			$parts[$label]=array('name'=>$i);
		}
		//1b. Now iterate through the headers again, trying to find the column numbers for each component's pH, amount, units 
		for($i=1;$i<100;$i++){ //1 because we know 0 is "reagent number" or similar
			if(!isset($secondRow[$i])){ break; }
			$label=trim($secondRow[$i]);
			if(empty($label)){ break; }
			if(false!==strpos($label,'[')){
				if('] units'==substr($label,-7)){
					$parts[substr($label,1,-7)]['unit']=$i;
				} else if(']'==substr($label,-1)){
					$parts[substr($label,1,-1)]['amount']=$i;
				} 
			} else if(false!==strpos($label,'pH_')){
				$parts[substr($label,3)]['pH']=$i;
			} else if('pH'==$label){
				//This is a pain. The label says nothing about what this is the pH *of*.
				//Work backward from this column to find something that's definitely a name/role
				for($j=0;$j<$i;$j++){
					$otherCol=$i-$j;
					$otherLabel=trim($secondRow[$otherCol]);
					if(']'==substr($otherLabel,-1)){
						$parts[substr($otherLabel,1,-1)]['pH']=$i;
						break;
					}
				}
			}
		}
		/*
		 * Step 2. Generate human-friendly representations of the conditions 
		 * Now we know which column represents what, go through each row in the file and concatenate columns in the right order.
		 */
		$fileContent=array_slice($fileContent,2); //Skip the first two rows, they are headers
		$conditions=array();
		$rowLabels=implode('', static::$rowLabels);
		foreach($fileContent as $row){
			$row=str_getcsv($row, $delimiter);
			//Row and column
			$wellLabel=trim($row[0]);
			if(empty($wellLabel)){ continue; }
			$rowLabel=substr($wellLabel,0,1);
			$rowNumber=stripos($rowLabels,$rowLabel);
			$rows=max($rows,$rowNumber);
			$columnNumber=(int)(substr($wellLabel,1));
			$cols=max($cols,$columnNumber);
			//Text description
			$labelParts=array();
			foreach($parts as $k=>$v){
				if(!isset($row[$v['amount']]) || !isset($row[$v['unit']]) || !isset($row[$v['name']])){
					continue;
				}
				$str=trim($row[$v['amount']].$row[$v['unit']].' '.$row[$v['name']]);
				if(!empty($str)){
					if(isset($v['pH']) && !empty($row[$v['pH']]) && false==strpos($str,'pH')){
						$str.=', pH '.$row[$v['pH']];
					}
					$labelParts[]=$str;
				}
			}
			$label=implode('; ',$labelParts);
			$condition=array(
					'description'=>$label,
					'screenid'=>$screenId,
					'projectid'=>$request['projectid'],
					'row'=>$rowNumber,
					'col'=>$columnNumber
			);
			$conditions[]=$condition;
		}
		
		//Now attempt to create all the conditions. Can't do it earlier because we didn't know how many rows/cols.
		foreach($conditions as $c){
			$wellNumber=1*((($c['row']-1)*$cols)+$c['col']);
			$c['wellnumber']=$wellNumber;
			$c['name']='screen'.$screenId.'_well'.str_pad($wellNumber, 3, '0', STR_PAD_LEFT);
			screencondition::create($c);
		}
		
		//and return the response from creating the screen itself
		return $ret;
		
	}
	
	/**
	 * Parses a CSV file with just well and condition into the database
	 * A1,Condition
	 * A01,Condition
	 * 1,Condition - assumes numbering A1-A12,B1-B12, etc.
	 * @param unknown $fileContent
	 * @param unknown $request
	 * @throws BadRequestException if file does not contain exactly 96 conditions or any well ID is outside 12x8
	 */
	public static function parseWellNameAndConditionToDatabase($fileContent,$request){
		//Public because installer uses it.
		
		//Try to create the screen before parsing its conditions
		$ret=parent::create($request,'screen');
		$screenId=$ret['created']['id'];
		
		//We'll populate this from the CSV
		$conditions=array(
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
		);
		$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		$delimiter=',';
		$firstRow=str_getcsv($fileContent[0], $delimiter);
		if(2>count($firstRow)){
			$delimiter=';';
			$firstRow=str_getcsv($fileContent[0], $delimiter);
			if(2>count($firstRow)){
				throw new BadRequestException('Could not parse CSV file, tried comma and semicolon as delimiters');
			} else if(2<count($firstRow)){
				throw new BadRequestException('Expected only two columns: Well identifier, and condition');
			}
		}
		foreach($fileContent as $row){
			$row=str_getcsv($row, $delimiter);
			if(empty($row) || empty($row[0])){ continue; } //blank line at end of file
			if('Well'==$row[0]){ continue; } //header row
			
			//Row and column
			$colNumber=0;
			$rowNumber=0;
			$wellLabel=trim($row[0]);
			$condition=trim($row[1]);
			if(preg_match('/^[A-H]\d\d?$/',$wellLabel)){
				$rowLabel=substr($wellLabel, 0, 1);
				$colNumber=(int)(substr($wellLabel,1));
				$rowNumber=array_search($rowLabel, static::$rowLabels);
				$wellNumber=(12*($rowNumber-1))+$colNumber;
			} else {
				$wellNumber=(int)$wellLabel;
				$colNumber=$wellNumber%12;
				if(0==$colNumber){ $colNumber=12; }
				$rowNumber=ceil($wellNumber/12);
			}
			if($rowNumber<1 || $rowNumber>8 || $colNumber<1 || $colNumber>12){
				throw new BadRequestException('Well identifier '.$wellLabel.' not recognised or out of bounds');
			}
			$conditions[$rowNumber-1][$colNumber-1]=$condition;						
		}
		for($r=1;$r<=8;$r++){
			for($c=1;$c<=12;$c++){
				$description=$conditions[$r-1][$c-1];
				if(empty($description)){
					throw new BadRequestException('No condition found for well '.self::$rowLabels[$r].$c);
				}
				$wellNumber=(12*($r-1))+$c;
				$condition=array(
						'name'=>'screen'.$screenId.'_well'.str_pad($wellNumber, 3, '0', STR_PAD_LEFT),
						'description'=>$description,
						'wellnumber'=>$wellNumber,
						'screenid'=>$screenId,
						'projectid'=>$request['projectid'],
						'row'=>$r,
						'col'=>$c
				);
				screencondition::create($condition);
			}
		}
		
		//Lastly, return the screen itself
		return $ret;
		
	}
	
}


?>